/* This file is part of ksirc
   Copyright (c) 2001 Malte Starostik <malte@kde.org>
             (c) 2002 Simon Hausmann <hausmann@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include "markup.h"

#include <qregexp.h>

using namespace IRC;

QString Markup::tagError( const QString &input )
{
    // ###
    return input;
}

QString Markup::tagInformation( const QString &input )
{
    // ###
    return input;
}

QString Markup::tagAction( const QString &input )
{
    // ### tagImage
    return input;
}

QString Markup::tagBold( const QString &input )
{
    return "<b>" + input + "</b>";
}

QString Markup::tagUnderlined( const QString &input )
{
    return "<u>" + input + "</u>";
}

QString Markup::tagImage( const QString &input, const QString &imageName )
{
    return QString::fromLatin1( "<img src=\"%1\" />" ).arg( imageName ) + input;
}

QString Markup::tagColorCodes( const QString &input )
{
    return Markup( input ).parseColorCodesToMarkup();
}

Markup::Markup( const QString &input )
    : m_input( input )
{
}

QString Markup::parseColorCodesToMarkup()
{
    QString res;
    m_tags.clear();
    m_attributes.clear();

    for ( unsigned int i = 0; i < m_input.length(); )
    {
        QChar ch = m_input[ i++ ];

        //if ( ch.latin1() == 0x03 || ch == '~' && i < m_input.length() )
        // no ksirc escape codes for now.
        if ( ch.latin1() == 0x03 && i < m_input.length() )
        {
            QChar next = m_input[ i++ ];
            if ( next.latin1() >= 0x30 && next.latin1() <= 0x39 )
            {
                int fg = -1, len = 0;
                int bg = -1;
                QRegExp colorRex( "^[0-9]+" );
                if ( colorRex.search( m_input.mid( --i ) ) >= 0 )
                {
                    len = colorRex.matchedLength();
                    fg = m_input.mid( i, len ).toInt();
                    i += len;
                }
                if ( m_input[ i ] == ',' )
                {
                    if ( colorRex.search( m_input.mid( ++i ) ) >= 0 )
                    {
                        len = colorRex.matchedLength();
                        bg = m_input.mid( i, len ).toInt();
                        i += len;
                    }
                }
                QColor c = ircColor( fg );
                if ( c.isValid() )
                    res += pushTag( "font", QString( "color=\"%1\"" ).arg( c.name() ) );
                //else
                //        res += popTag( "font" );

                c = ircColor( bg );
                if ( c.isValid() )
                    res += pushTag( "font", QString( "bgcolor=\"%1\"" ).arg( c.name() ) );
            }
            else if ( ch.latin1() == 0x03 )
                res += popTag( "font" );
            /*
            else if ( ch == '~' )
            {
                switch ( next )
                {
                case 'c':
                    res += popTag( "font" );
                    break;
                case 'C':
                    res += popAll();
                    break;
                case 'r': break;
                case 's': break;
                case 'b':
                    res += toggleTag( "b" );
                    break;
                case 'u':
                    res += toggleTag( "u" );
                    break;
                case 'n':
                //    res += pushTag( "font", QString( "color=\"%1\"" ).arg( ksopts->nickForeground.name() ) );
                    break;
                case 'o':
                //    res += pushTag( "font", QString( "color=\"%1\"" ).arg( ksopts->ownNickColor.name() ) );
                    break;
                case '~':
                    res += ch;
                }
            }
            */
        }
        else
            res += ch;

    }

    res.append( popAll() );
    return res;
}

QString Markup::pushTag( const QString &tag, const QString &attributes )
{
    QString res;

    m_tags.push( tag );

    if ( !m_attributes.contains( tag ) )
        m_attributes.insert( tag, attributes );
    else if ( !attributes.isEmpty() )
        m_attributes.replace( tag, attributes );

    res.append( "<" + tag );

    if ( !m_attributes[ tag ].isEmpty() )
        res.append( " " + m_attributes[ tag ] );

    return res + ">";
}

QString Markup::popTag( const QString &tag )
{
    if ( !m_tags.contains(tag ) )
        return QString::null;

    QString res;
    QValueStack<QString> savedTags;

    while ( m_tags.top() != tag )
    {
        savedTags.push( m_tags.pop() );
        res.append( "</" + savedTags.top() + ">" );
    }

    res.append( "</" + m_tags.pop() + ">" );

    m_attributes.remove( tag );

    while ( !savedTags.isEmpty() )
        res.append( pushTag( savedTags.pop() ) );

    return res;
}

QString Markup::toggleTag( const QString &tag )
{
    return m_attributes.contains( tag ) ? popTag( tag ) : pushTag( tag );
}

QString Markup::popAll()
{
    QString res;

    while ( !m_tags.isEmpty() )
        res.append( "</" + m_tags.pop() + ">" );

    m_attributes.clear();
    return res;
}

QColor Markup::ircColor( int code )
{
    if ( code < 0 || code >= 16 )
        return QColor();

//    return ksopts->ircColors[code];
// ### clean me up
    QColor ircColors[ 16 ];
    ircColors[0] = Qt::white;
    ircColors[1] = Qt::black;
    ircColors[2] = Qt::darkBlue;
    ircColors[3] = Qt::darkGreen;
    ircColors[4] = Qt::red;
    ircColors[5] = QColor ( 165, 42, 42 );
    ircColors[6] = Qt::darkMagenta;
    ircColors[7] = QColor ( 255, 255, 0 );
    ircColors[8] = Qt::yellow;
    ircColors[9] = Qt::green;
    ircColors[10] = Qt::darkCyan;
    ircColors[11] = Qt::cyan;
    ircColors[12] = Qt::blue;
    ircColors[13] = QColor ( 255, 192, 203 );
    ircColors[14] = Qt::gray;
    ircColors[15] = Qt::lightGray;

    return ircColors[ code ];
}


/* vim: et sw=4 ts=4
 */
