/* This file is part of the KDE project
   Copyright (C) 2002 Simon Hausmann <hausmann@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include "windowforwarder.h"
#include "frontend.h"
#include "window.h"

#include <assert.h>

using namespace IRC;

WindowForwarder::WindowForwarder( Frontend *parent, const char *name )
    : MessageForwarder( parent, name ), m_frontend( parent )
{
    registerForwardMapping( PrivmsgMessage::name(), 0 );
    registerForwardMapping( NoticeMessage::name(), 0 );
    registerForwardMapping( TopicMessage::name(), 0 );
    registerForwardMapping( JoinMessage::name(), 0 );
    registerForwardMapping( PartMessage::name(), 0 );
    registerForwardMapping( numericReplyToString( RPL_TOPIC ), 1 );
    registerForwardMapping( numericReplyToString( RPL_NOTOPIC ), 1 );
}

void WindowForwarder::registerWindow( Window *window )
{
    registerMessageReceiver( window->channel(), window );
}

void WindowForwarder::handlePrivmsgMessage( const IRC::PrivmsgMessage &msg )
{
    forwardMessage( msg );
}

void WindowForwarder::handleNoticeMessage( const IRC::NoticeMessage &msg )
{
    forwardMessage( msg );
}

void WindowForwarder::handleTopicMessage( const IRC::TopicMessage &msg )
{
    forwardMessage( msg );
}

void WindowForwarder::handleJoinMessage( const IRC::JoinMessage &msg )
{
    forwardMessage( msg );
}

void WindowForwarder::handlePartMessage( const IRC::PartMessage &msg )
{
    forwardMessage( msg );
}

void WindowForwarder::handleNumericMessage( const IRC::Message &msg )
{
    forwardMessage( msg );
}

void WindowForwarder::forwardMessage( const QString &_key, const IRC::Message &msg )
{
    QString key = _key.lower();

    // for non-channels assume a private chat and use the nick from the message prefix
    if ( key[ 0 ] != '#' ) {
        key = msg.sender().nick();
        if ( key.isEmpty() ) {
            emit unhandledMessage( msg );
            return;
        }
    }

    Window *wnd = m_frontend->windowForChannel( key, Frontend::CreateIfNotExisting );
    assert( wnd );
    MessageForwarder::forwardMessage( key, msg );
}

#include "windowforwarder.moc"

/* vim: et sw=4 ts=4
 */
